/* ws2812b.c */

#include <stm32f10x.h>
#include <stm32f10x_rcc.h>
#include <stm32f10x_gpio.h>
#include <stm32f10x_tim.h>
#include <stm32f10x_dma.h>
#include <misc.h>

#include "ws2812b.h"

#define bitValue0 29
#define bitValue1 61

uint8_t ws2812b_transferComplete = 1;

void init_timers(void) {
    GPIO_InitTypeDef	GPIO_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;
    // PA1     2  2.
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);

    GPIO_StructInit(&GPIO_InitStructure);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed	= GPIO_Speed_2MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
    TIM_OCInitTypeDef TIM_OCInitStructure;

    //   
    RCC_APB1PeriphClockCmd( RCC_APB1Periph_TIM2, ENABLE );
    // configure timer
    // PWM frequency = 50 hz with 24 ,000 ,000 hz system clock
    //  24 ,000 ,000/480 = 50 ,000
    // 100 ,000/100 = 50

    TIM_TimeBaseStructInit(&TIM_TimeBaseStructure );
    TIM_TimeBaseStructure.TIM_Prescaler = 0; // 72 MHz
    TIM_TimeBaseStructure.TIM_Period = 90 - 1; // 1,242 .  1,25 
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInit (TIM2, &TIM_TimeBaseStructure );
    // CRR == 29 // 0.4   "0"
    // CRR == 61 // 0.85  "1"

    // 24 :
    // = 30 (1.251 );
    //CCR 0.4  = 10 (0,417 )
    //CCR 0.85  = 21 (0,875 )

// PWM1 Mode configuration : Channel2
//Edge - aligned ; not single pulse mode
    TIM_OCStructInit (&TIM_OCInitStructure );
    TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
    //     
    TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
    TIM_OC2Init(TIM2, &TIM_OCInitStructure );

    // .   CCR2      .
    TIM_OC2PreloadConfig(TIM2, TIM_OCPreload_Enable);

    TIM2->CNT = 0;
    TIM2->CCR2 = 0;

    TIM_DMACmd(TIM2, TIM_DMA_CC2, ENABLE);  //  DMA

    TIM_ITConfig(TIM2, TIM_IT_CC2, ENABLE); //   

// Enable Timer
    //TIM_Cmd(TIM2, ENABLE ); //   

    //TIM2->CCER |= 0x10;     //   ( TIM_OutputState_Enable )

    NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;        // DISABLE
	NVIC_Init(&NVIC_InitStructure);

}

// tbuf      .
//     .   :
// 1.  (          ).
//    24    .     
// 2.  .    GBR   .
// 3.    .        .
//   (  )
//       tbuf.
//
//        GRB     .

void dma_initial(const void *tbuf){

	DMA_InitTypeDef DMA_InitStructure;
	DMA_Channel_TypeDef *txChan;

	NVIC_InitTypeDef NVIC_InitStructure;

	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

	txChan = DMA1_Channel7;      //  2  2

	DMA_DeInit(txChan);

	DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)(&(TIM2->CCR2));

	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;

	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_BufferSize = 1;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Normal; //DMA_Mode_Circular;
	DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;

	//
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)tbuf;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
	DMA_Init(txChan, &DMA_InitStructure);

	//       DMA
	DMA_ITConfig(txChan, DMA_IT_TC, ENABLE);

	// Enable channel
	//DMA_Cmd(txChan, ENABLE);

	// Wait for completion
	//while (DMA_GetFlagStatus(DMA1_FLAG_TC7) == RESET);

	// Disable channels
	//DMA_Cmd(txChan, DISABLE);

	NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel7_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;              // DISABLE
	NVIC_Init(&NVIC_InitStructure);

}

void ws2812b_init(const void *tbuf){
	uint8_t i;

	for ( i = 0; i < 48; i++ ) {
		((uint8_t*)tbuf)[i] = 0; //  
	}

	//  .  .
	((uint8_t*)tbuf)[48 + NumberOfLEDS * 24] = 0;

	init_timers();
	dma_initial(tbuf);
}

void ws2812b_showLEDsArray(void){

	ws2812b_transferComplete = 0;

	DMA1_Channel7->CNDTR = 49 + NumberOfLEDS * 24;
	// Enable DMA channel
	DMA_Cmd(DMA1_Channel7, ENABLE);


TIM2->EGR |= 0x01; //      

	// Enable Timer
	TIM_Cmd(TIM2, ENABLE);

	while(!ws2812b_transferComplete);

}

// GRB      
void ws2812b_fillLEDsArray(uint8_t *tbuf, struct color * Colr){

uint8_t colors8bit[8], threeColors[3];
uint8_t bit;
uint8_t * data = tbuf + 48;
uint8_t i, z;
uint16_t y;

 if( Colr != (struct color *)0 ){
	 threeColors[0] = Colr->G;
	 threeColors[1] = Colr->R;
	 threeColors[2] = Colr->B;
 }else {
	 threeColors[0] = 0;
	 threeColors[1] = 0;
	 threeColors[2] = 0;
 }

 for ( z = 0; z < 3; z++ ){

	 bit = 0x80;

	 for( i = 0; i < 8; i++ ){

		 if( threeColors[z] & bit ){
			 colors8bit[i] = bitValue1;
		 }else {
			 colors8bit[i] = bitValue0;
		}

		 bit = bit >> 1;
	 }

	 for ( y = 0; y < NumberOfLEDS; y++ ){

		 for ( i = 0; i < 8; i++){
			 *(data + z*8 + y*24 + i) = colors8bit[i];
		 }
	 }

 }

}

void ws2812b_changeLed(uint8_t *tbuf, uint8_t on, uint16_t number, struct color * Colr){

uint8_t colors8bit[8], threeColors[3];
uint8_t bit;
uint8_t * data = tbuf + 48;
uint8_t i, z;

if ( number >= NumberOfLEDS )
	return;

if ( on ){
 threeColors[0] = Colr->G;
 threeColors[1] = Colr->R;
 threeColors[2] = Colr->B;
}else{
	threeColors[0] = 0;
	threeColors[1] = 0;
	threeColors[2] = 0;
}
 for ( z = 0; z < 3; z++ ){

	 bit = 0x80;

	 for( i = 0; i < 8; i++ ){

		 if( threeColors[z] & bit ){
			 colors8bit[i] = bitValue1;
		 }else {
			 colors8bit[i] = bitValue0;
		}

		 bit = bit >> 1;
	 }

	 for ( i = 0; i < 8; i++){
		 *(data + number*24 + z*8 + i) = colors8bit[i];
	 }


 }

}

void DMA1_Channel7_IRQHandler(void){

	if ( DMA1->ISR & DMA1_IT_TC7 ){
		DMA1->IFCR |= (DMA1_IT_TC7 | DMA1_IT_HT7);

		ws2812b_transferComplete = 1;

	}

}

void TIM2_IRQHandler(void){

	if( TIM2->SR & TIM_FLAG_CC2 ){
		TIM2->SR &= ~TIM_FLAG_CC2; // rc_w0

		if ( ws2812b_transferComplete == 1 ){
			// Disable Timer
			TIM_Cmd(TIM2, DISABLE);

			DMA_Cmd(DMA1_Channel7, DISABLE);

		}
	}
}
